import tensorflow as tf
import tf2onnx
import onnx


class EasyExporter:
    def __init__(self):
        pass

    def export_to_onnx(self, name, path):
        """
        Function to export from keras to onnx.
        Docs: https://github.com/Gabrli/EasyCNN---docs

        :param name: name for new model
        :param path: path to old model
        """
        model = tf.keras.models.load_model(path)
        model.output_names=['output']
        input_shape = model.input_shape[1:]
        spec = (tf.TensorSpec((None, *input_shape), tf.float32, name="input"),)
        output_path = name + ".onnx"
        
        model_proto, _ = tf2onnx.convert.from_keras(model, input_signature=spec, opset=13, output_path=output_path)
        onnx.save_model(model_proto, output_path)
    




from tensorflow.keras.applications import (
ResNet50, ResNet152, ResNet50V2, ResNet101, ResNet101V2, ResNet152V2,
VGG16, VGG19, InceptionV3, MobileNet, MobileNetV2, MobileNetV3Small,
InceptionResNetV2, MobileNetV3Large, Xception
)

class Preset:
    """
        Ready to use preset of keras application.
        Docs: https://github.com/Gabrli/EasyCNN---docs
    """
    def __init__(self):
        self.models = {
            "resnet50": ResNet50,
            "resnet152": ResNet152,
            "resnet50v2": ResNet50V2,
            "resnet101": ResNet101,
            "resnet101v2": ResNet101V2,
            "resnet152v2": ResNet152V2,
            "vgg16": VGG16,
            "vgg19": VGG19,
            "inceptionv3": InceptionV3,
            "inceptionresnetv2": InceptionResNetV2,
            "mobilenet": MobileNet,
            "mobilenetv2": MobileNetV2,
            "mobilenetv3small": MobileNetV3Small,
            "mobilenetv3large": MobileNetV3Large,
            "xception": Xception
        }

    def get_preset(self, model_name, classes, x=32, y=32, r=3):
        """
        Returns a preset model based on the model name.
        
        List of avaliable presets: https://github.com/Gabrli/EasyCNN---docs/blob/main/docs/presets/presets-list.md
        Full Docs: https://github.com/Gabrli/EasyCNN---docs
        
        :param model_name: Name of the model (e.g., 'resnet50', 'vgg16').
        :param classes: Number of output classes.
        :param x: Width of the input image.
        :param y: Height of the input image.
        :param r: Number of channels in the input image.
        :return: A Keras model instance.
        """
        model = self.models.get(model_name.lower())
        if not model:
            raise ValueError(f"Model '{model_name}' is not supported.")
        return model(include_top=False, input_shape=(x, y, r), pooling='avg', classes=classes, weights='imagenet')


from tensorflow.keras import layers, models
import numpy as np
import cv2 as cv
class LiteCNN:
    """
    Docs: https://github.com/Gabrli/EasyCNN---docs
    """
    def __init__(self, input_shape=[32,32,3]):
        """
        :param input_shape: array if three values x, y, r
        """
        self.model = models.Sequential()
        self.input_shape = input_shape
    def add(self,layers):
        """
        Function to adding custom layers by user.
        :param layers: your additional layers
        """
        self.model.add(layers)
    def add_conv(self, filters, kernel_size, activation='relu'):
        if self.model.layers == 0:
            self.model.add(layers.Conv2D(filters, (kernel_size, kernel_size), activation=activation, input_shape=(self.input_shape[0], self.input_shape[1], self.input_shape[2])))
        self.model.add(layers.Conv2D(filters, (kernel_size, kernel_size), activation=activation))
    def add_max_pool(self, pool_size):
        self.model.add(layers.MaxPooling2D((pool_size, pool_size)))
    def add_flatten(self):
        self.model.add(layers.Flatten())
    def add_dense(self, filters, activation='relu'):
        self.model.add(layers.Dense(filters, activation=activation))
    def add_dropout(self, value = 0.3):
        self.model.add(layers.Dropout(value))
    def add_batch_normalization(self):
        self.model.batchNormalization()
    def add_r_flip(self, value="horizontal"):
        self.model.add(layers.RandomFlip(value))
    def add_r_rotate(self, value=0.2):
        self.model.add(layers.RandomRotation(value))
    def add_r_zoom(self, value=0.2):
        self.model.add(layers.RandomZoom(value))
    def compile(self, optimizer='adam', loss='sparse_categorical_crossentropy', metrics=['accuracy']):
        self.model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    def train(self, x, y, test_x, test_y, epochs=10, batch_size=32):
        return self.model.fit(x, y, epochs=epochs, batch_size=batch_size, validation_data=(test_x, test_y))    
    def save(self, path):
        self.model.save(path)
    def load(self, path):
        self.model = models.load_model(path)

        
    def predict(self, x):
        img = cv.imread(x)
        img = cv.resize(img, (32,32))
        img = cv.cvtColor(img, cv.COLOR_BGR2RGB)
        img_np = np.array(img)
        img_np = img / 255
        img_np = np.expand_dims(img_np, axis=0)
        predict = self.model.predict(img_np)
        predict_class = np.argmax(predict)
        return predict_class



import matplotlib.pyplot as plt
class TrainingVisualizer:
    
    def plot_training(self, history):
        """
        Function to dispaly in graphic format model training process.
        Docs: https://github.com/Gabrli/EasyCNN---docs
        
        :param history: value of training process history

        """
        plt.figure(figsize=(12, 5))
        def draw(parameter, index):
           plt.subplot(1,2, index)
           plt.plot(history.history[parameter], label=f'Train {parameter}')
           plt.plot(history.history[f'val_{parameter}'], label=f'Val {parameter}')
           plt.title(f'{parameter} over epochs')
           plt.legend() 

        draw('loss', 1)
        draw('accuracy', 2)
        plt.show()
    


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="litecnn",
    version="1.0.3",
    description="LiteCNN: Intuitive Python library for creating, training and visualizing convolutional neural networks. Features simplified CNN layer definition, automated training workflows, model visualization, and seamless Keras-to-ONNX conversion. Includes 15 pre-configured popular models for immediate use.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    project_urls={
        "Documentation": "https://github.com/Gabrli/EasyCNN---docs",
        "Source Code" :"https://github.com/Gabrli/easyCNN",
    },
    keywords="deep-learning, cnn, neural-networks, tensorflow, keras, machine-learning, ai, computer-vision, image-processing, model-training, visualization, easy-to-use, python, convolutional-networks",
    packages=find_packages(),
    author="Gabriel Wiśniewski",
    author_email="gabrys.wisniewski@op.pl",
    classifiers=[  
        "Programming Language :: Python :: 3",  
        "License :: OSI Approved :: MIT License",  
        "Operating System :: OS Independent", 
    ],
     install_requires=[
        "tensorflow",
        "numpy",
        "matplotlib",
        "opencv-python",
        "tf2onnx",
        "onnx"
    ],
    entry_points={
        "console_scripts":[
            "litecnn-core = litecnn:LiteCNN",
            "litecnn-preset = litecnn:Preset",
            "litecnn-visualizer = litecnn:TrainingVisualizer",
            "litecnn-exporter = litecnn:EasyExporter",
            
        ]
    }
)